"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require('chalk');
const ejs = require("ejs");
const fs = require("fs");
const jsBeautify = require("js-beautify");
const mkdirp = require("mkdirp");
const path = require("path");
const tsconfig = require('../../templates/tsconfig.e2e.json');
const util_1 = require("util");
const mkdirpAsync = util_1.promisify(mkdirp);
const readFileAsync = util_1.promisify(fs.readFile.bind(fs));
const writeFileAsync = util_1.promisify(fs.writeFile.bind(fs));
const cukeJsonFormatter = `json:./reports/cucumber_report.json`;
let beforeLaunchExpression;
let formatExpression;
let jasmineReportExpression;
let logExpression;
let mochaReportExpression;
let onCompleteExpression;
let onPrepareExpression;
let seleniumAddressExpression;
let seleniumServerJarExpression;
let transpilerExpression;
const tsconfigTypes = [];
function existsAsync(filePath) {
    let fulfill = null;
    const promise = new Promise((x) => {
        fulfill = x;
    });
    fs.access(filePath, (err) => {
        fulfill(!err);
    });
    return promise;
}
exports.existsAsync = existsAsync;
function createDirectory(filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (filepath.indexOf('*') > -1) {
            filepath = filepath.substring(0, filepath.indexOf('*'));
        }
        const dirname = path.join(process.cwd(), filepath);
        try {
            if (!(yield existsAsync(dirname))) {
                mkdirpAsync(dirname);
            }
        }
        catch (err) {
            throw new Error(err.toString());
        }
    });
}
function createTSconfigfile(options) {
    return __awaiter(this, void 0, void 0, function* () {
        tsconfig.compilerOptions.types = tsconfig.compilerOptions.types.concat(options);
        try {
            yield writeFileAsync(path.join(process.cwd(), './tsconfig.e2e.json'), JSON.stringify(tsconfig, null, 4));
        }
        catch (err) {
            throw new Error(err.toString());
        }
    });
}
function createTSLintFile() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield writeFileAsync(path.join(process.cwd(), './tslint.json'), JSON.stringify(tsconfig, null, 4));
        }
        catch (err) {
            throw new Error(err.toString());
        }
    });
}
function createESLintRCFile() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield writeFileAsync(path.join(process.cwd(), './.eslintrc.json'), JSON.stringify(tsconfig, null, 4));
        }
        catch (err) {
            throw new Error(err.toString());
        }
    });
}
function createConfigFile(answers) {
    return __awaiter(this, void 0, void 0, function* () {
        if (answers.backend.indexOf('local machine') > -1) {
            if (!answers.drivers && !answers.webdriver) {
                seleniumServerJarExpression = `seleniumServerjar: '${answers.seleniumJar}',`;
                seleniumAddressExpression = `seleniumAddress: '${answers.seleniumAddress}',`;
            }
            if (answers.webdriver) {
                seleniumAddressExpression = `seleniumAddress: '${answers.seleniumAddress}',`;
            }
        }
        else if (answers.backend.indexOf('remote selenium server') > -1) {
            seleniumAddressExpression = `seleniumAddress: '${answers.host}',`;
        }
        if (answers.framework === 'jasmine') {
            if (answers.transpilerType === 'typescript') {
                tsconfigTypes.push('jasmine', 'jasminewd2');
                beforeLaunchExpression = `require('ts-node').register({
            project: './tsconfig.e2e.json'
         });`;
            }
            if (answers.reportType === 'spec') {
                jasmineReportExpression = `const {SpecReporter} = require('jasmine-spec-reporter');`;
                onPrepareExpression = `jasmine.getEnv().addReporter(new SpecReporter({
            spec: {
              displayStacktrace: true
            }
          }));
          `;
            }
            else if (answers.reportType === 'html') {
                jasmineReportExpression =
                    `const HtmlScreenshotReporter = require('protractor-jasmine2-screenshot-reporter');`;
                onPrepareExpression = `jasmine.getEnv().addReporter(new HtmlScreenshotReporter({
            dest: 'target/screenshots',
            filename: 'protractor_jasmine_report.html'
          }));
              `;
            }
        }
        if (answers.framework === 'mocha') {
            if (answers.transpilerType === 'typescript') {
                tsconfigTypes.push('mocha');
                transpilerExpression = `'ts:ts-node/register'`;
            }
            else if (answers.transpilerType === 'coffeescript') {
                transpilerExpression = `'coffee:coffeescript/register'`;
            }
            if (answers.reportType === 'html') {
                mochaReportExpression = `'mochawesome',
        reporterOptions : {
            reportDir: './reports',
            reportFileName: 'protractor_mocha_report',
            enableCharts: true
        }`;
            }
            else {
                mochaReportExpression = answers.reportType;
            }
        }
        if (answers.framework === 'cucumber') {
            if (answers.transpilerType === 'typescript') {
                tsconfigTypes.push('cucumber');
                transpilerExpression = `'ts:ts-node/register'`;
            }
            else if (answers.transpilerType === 'coffeescript') {
                transpilerExpression = `'coffee:coffeescript/register'`;
            }
            formatExpression = answers.cucumberReportType;
            if (answers.cucumberReportType === 'json') {
                formatExpression = cukeJsonFormatter;
            }
            if (answers.cucumberReportType === 'html') {
                formatExpression = cukeJsonFormatter;
                onCompleteExpression = `const cucumberReporterOptions = {
                  theme: 'bootstrap',
                  jsonFile: './reports/cucumber_report.json',
                  output: process.cwd() + './reports/cucumber_reporter.html',
                  reportSuiteAsScenarios: true
                  };
          reporter.generate(cucumberReporterOptions);
    `;
            }
        }
        try {
            if (answers.specPath) {
                yield createDirectory(answers.specs);
            }
            if (answers.featurePath) {
                yield createDirectory(answers.specs);
                yield createDirectory(answers.stepDefinitions);
            }
            if (answers.transpilerType === 'typescript') {
                yield createTSconfigfile(tsconfigTypes);
            }
            if (answers.linter === 'tslint') {
                yield createTSLintFile();
            }
            if (answers.linter === 'eslint-plugin-protractor') {
                yield createESLintRCFile();
            }
        }
        catch (err) {
            throw new Error(err);
        }
        if (answers.logging === 'error') {
            logExpression = answers.logging.toUpperCase();
        }
        else if (answers.logging === 'warn') {
            logExpression = answers.logging.toUpperCase();
        }
        else if (answers.logging === 'debug') {
            logExpression = answers.logging.toUpperCase();
        }
        else {
            logExpression = 'INFO';
        }
        if (answers.createReportPath) {
            yield createDirectory(answers.reportPath);
        }
        try {
            const templateData = yield readFileAsync(path.join(__dirname, '../../templates/protractor.conf.ejs'), 'utf8');
            const renderedFile = ejs.render(templateData, {
                answers,
                beforeLaunchExpression,
                formatExpression,
                jasmineReportExpression,
                logExpression,
                mochaReportExpression,
                onCompleteExpression,
                onPrepareExpression,
                seleniumAddressExpression,
                seleniumServerJarExpression,
                transpilerExpression,
            });
            yield writeFileAsync(path.join(process.cwd(), './protractor.conf.js'), jsBeautify(renderedFile));
            console.log(`
${chalk.green('Configuration file was created successfully!')}
${chalk.green('To run your tests, execute:')}
${chalk.green('$ protractor protractor.conf.js')}
`);
        }
        catch (err) {
            throw new Error(err.toString());
        }
    });
}
exports.createConfigFile = createConfigFile;
//# sourceMappingURL=fileHelper.js.map