"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fileHelper_1 = require("../helpers/fileHelper");
const { spawn } = require('child_process');
const chalk = require('chalk');
const path = require("path");
function checkPackageJson() {
    return __awaiter(this, void 0, void 0, function* () {
        const dir = path.resolve(process.cwd());
        const pkgJson = path.join(dir, 'package.json');
        if (!(yield (fileHelper_1.existsAsync(pkgJson)))) {
            console.log(chalk.red('\nCould not find a package.json file! ') +
                chalk.green('Creating package.json...'));
            const initPkg = spawn('npm', ['init', '--yes'], { stdio: 'pipe' });
            let stderr = '';
            initPkg.stderr.on('data', (data) => {
                stderr += data;
            });
            initPkg.on('close', (code) => {
                if (code !== 0) {
                    console.log(chalk.red('\nUnable to create package.json! ' +
                        `${stderr}`));
                }
            });
        }
    });
}
exports.checkPackageJson = checkPackageJson;
function installPkgs(des, packages, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            const args = ['--loglevel=error'];
            if (opts.save) {
                args.push('-S');
            }
            if (opts.saveDev) {
                args.push('-D');
            }
            if (opts.global) {
                args.push('-g');
            }
            const cliArgs = ['i'].concat(args, packages);
            process.stdout.write('\n' + chalk.yellow(des) + '\n');
            const install = spawn('npm', cliArgs, { stdio: 'inherit' });
            install.on('error', (err) => {
                if (err) {
                    reject(err);
                }
            });
            install.on('exit', (code) => {
                if (code === 0) {
                    resolve('successfully installed packages!');
                }
            });
        });
    });
}
exports.installPkgs = installPkgs;
function unInstallPkgs(des, packages, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            const args = ['--loglevel=error'];
            if (opts.save) {
                args.push('-S');
            }
            if (opts.saveDev) {
                args.push('-D');
            }
            if (opts.global) {
                args.push('-g');
            }
            const cliArgs = ['uninstall'].concat(args, packages);
            process.stdout.write('\n' + chalk.yellow(des) + '\n');
            const install = spawn('npm', cliArgs, { stdio: 'inherit' });
            install.on('error', (err) => {
                if (err) {
                    reject(err);
                }
            });
            install.on('exit', (code) => {
                if (code === 0) {
                    resolve('successfully unInstalled packages!');
                }
            });
        });
    });
}
exports.unInstallPkgs = unInstallPkgs;
function updateWebdriver() {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            process.stdout.write(chalk.yellow('\nDownloading chrome, firefox & internet explorer drivers...\n'));
            const install = spawn('webdriver-manager', ['update', '--ie'], { stdio: 'inherit' });
            install.on('error', (err) => {
                if (err) {
                    reject(err);
                }
            });
            install.on('exit', (code) => {
                if (code === 0) {
                    resolve(`
${chalk.green('Protractor & Webdriver-Manager dependencies installed successfully!')}
${chalk.green('To start the selenium server, execute:')}
${chalk.green('$ webdriver-manager start')}
`);
                }
            });
        });
    });
}
exports.updateWebdriver = updateWebdriver;
//# sourceMappingURL=npmUtil.js.map